#! /usr/bin/env python
# coding: utf-8

import sys, irc.bot, random, string, unicodedata

if len(sys.argv) > 1:
    name = sys.argv[1]
else: name = "HangBot"
chans = ["##-tdct-jeux"]
server = "irc.freenode.org"

maxtries = 12
def asciize(word):
    return u"".join(x for x in unicodedata.normalize("NFKD",
        word) if x in string.ascii_letters+u"-").upper()

words,shortest,longest = {},200,0
f = open("/usr/share/dict/french", "r")
for line in f.readlines():
    real = line[:-1].decode("utf-8")
    word = asciize(real)
    words[word] = real
    word = word.replace("-", "")
    longest = max(longest, len(word))
    shortest = min(shortest, len(word))
f.close()

class Hangman(object):
    def __init__(self, bot, word):
        self.word = word
        self.errors = 0
        self.bot = bot
    
    def lettertry(self, letter):
        if letter not in self.word: self.errors += 1
        return self.errors > maxtries or self.current() == self.word
    
    def current(self):
        current = self.word # On remplace…
        for letter in u"ABCDEFGHIJKLMNOPQRSTUVWXYZ":
            if letter not in self.bot.letters:
                current = current.replace(letter, "_")
        return current
    
    def __str__(self):
        if self.errors > maxtries:
            return u"perdu ! C'était « "+words[self.word]+u" »"
        current = self.current()
        if current == self.word:
            return u"bravo ! C'était bien « "+words[self.word]+u" »"
        return u" ".join(current)+u" ["+str(self.errors)+u"/"+str(maxtries)+"]"

class Bot(irc.bot.SingleServerIRCBot):
    def __init__(self):
        irc.bot.SingleServerIRCBot.__init__(self, [(server, 6667)], name, name)
        self.hangmen,self.letters,self.errors = [],[],-1
    
    def on_welcome(self, c, e):
        for chan in chans:
            c.join(chan)
    
    def on_privmsg(self, c, e):
        nick = e.source.nick.decode()
        c.action(nick, u"câline "+nick)
    
    def on_pubmsg(self, c, e):
        if e.arguments[0][0] not in u"!%$§:;&£@#?":
            return # Pas une demande adressée au bot.
        command = e.arguments[0][1:].strip().rstrip().split()
        nick = e.source.nick.decode()
        if command[0].startswith("start"):
            self.launch(c, e.target, nick, command)
        elif command[0].startswith("draw"):
            self.draw(c, e.target, nick)
        elif command[0].startswith("show"):
            self.show(c, e.target, nick, True)
        elif len(command[0]) == 1 and command[0] in string.ascii_letters:
            self.lettertry(c, e.target, nick, command[0].upper())
        else: self.usage(c, e.target, nick)
    
    def lettertry(self, c, target, nick, letter):
        if letter in self.letters:
            c.privmsg(target, nick+u", lettre déjà jouée !")
            return # Bon, bah, c'est tout.
        self.letters.append(letter)
        for hangman in self.hangmen[:]:
            if hangman.lettertry(letter):
                self.hangmen.remove(hangman)
                if hangman.errors > maxtries:
                    self.errors = maxtries+1
            c.privmsg(target, nick+u", "+unicode(hangman))
        if self.errors <= maxtries:
            if len(self.hangmen) == 0: self.errors = -1
            else: self.errors = max(hangman.errors for hangman in self.hangmen)
        if len(self.hangmen) == 0:
            c.privmsg(target, nick+u", partie terminée !")
        else: self.show(c, target, nick, False)
    
    def launch(self, c, target, nick, command):
        if len(self.hangmen) > 0 and len(self.letters) > 0:
            c.privmsg(target, nick+u", on finit le jeu en cours d'abord !")
            return # Partie déjà démarrée, on ne lance pas.
        if len(command) >= 2: # Un nombre précisé…
            try: # Si c'est un entier.
                numof = int(command[1])
                if numof <= 0: # Oups.
                    raise ValueError
                elif numof > 25:
                    c.privmsg(target, nick+u", c'est un peu trop.")
                    c.privmsg(target, nick+u", sois gentil avec unicorn.")
                    return # Ouaip, on s'arrête là.
                if len(command) == 3:
                    smin = int(command[2])
                    smax = smin
                elif len(command) >= 4:
                    smin = int(command[2])
                    smax = int(command[3])
                else: smin,smax = shortest,longest
            except ValueError:
                self.usage(c, e.target, nick)
                return # Entier invalide.
        else: numof,smin,smax = 6,shortest,longest
        wordlist = [] # Allons-y.
        for word in words.keys():
            size = len(word.replace("-", ""))
            if size >= smin and size <= smax:
                wordlist.append(word)
        if len(wordlist) == 0:
            c.privmsg(target, nick+u", erreur : pas de mots trouvés.")
            return # Bon, bah, tant pis.
        elif len(wordlist) < numof:
            c.privmsg(target, nick+u", erreur : pas assez de mots trouvés.")
        for i in range(min(numof, len(wordlist))):
            word = random.choice(wordlist)
            self.hangmen.append(Hangman(self, word))
            wordlist.remove(word)
        self.letters,self.errors = [],0
        self.show(c, target, nick, True)
    
    def usage(self, c, target, nick):
        if len(self.hangmen) > 0:
            c.privmsg(target, nick+u", !lettre pour jouer une lettre")
            c.privmsg(target, nick+u", !show pour afficher l'état")
            c.privmsg(target, nick+u", !draw pour dessiner l'état")
            return # Voilà, c'est tout dans ce cas.
        c.privmsg(target, nick+u", usage : !start [X [Y [Z]]]")
        c.privmsg(target, nick+u", X indique le nombre"
                  +u" de pendus désirés (6 par défaut)")
        c.privmsg(target, nick+u", Y, s'il est précisé"
                  +u" seul, indique la taille des mots")
        c.privmsg(target, nick+u", Sinon, Y indique la"
                     +u" taille min et Z la taille max")
    
    def show(self, c, target, nick, words):
        if words: # On affiche les mots aussi.
            for hangman in self.hangmen:
                c.privmsg(target, nick+u", "+unicode(hangman))
        letters = list(u"ABCDEFGHIJKLMNOPQRSTUVWXYZ")
        for letter in self.letters:
            letters.remove(letter)
        c.privmsg(target, nick+u", ["+u",".join(letters)+u"]")
    
    def draw(self, c, target, nick):
        if self.errors >= 13:
            message = u"""
  _________ 
  | /   |   
  |/   (X)  
  |    /|\  
  |    / \  
  |         
__|_________"""
        elif self.errors == 12:
            message = u"""
  _________ 
  | /   |   
  |/   (O)  
  |    /|\  
  |    / \  
  |         
__|_________"""
        elif self.errors == 11:
            message = u"""
  _________ 
  | /   |   
  |/   (O)  
  |    /|\  
  |    /    
  |         
__|_________"""
        elif self.errors == 10:
            message = u"""
  _________ 
  | /   |   
  |/   (O)  
  |    /|\  
  |         
  |         
__|_________"""
        elif self.errors == 9:
            message = u"""
  _________ 
  | /   |   
  |/   (O)  
  |     |\  
  |         
  |         
__|_________"""
        elif self.errors == 8:
            message = u"""
  _________ 
  | /   |   
  |/   (O)  
  |     |   
  |         
  |         
__|_________"""
        elif self.errors == 7:
            message = u"""
  _________ 
  | /   |   
  |/   (O)  
  |         
  |         
  |         
__|_________"""
        elif self.errors == 6:
            message = u"""
  _________ 
  | /   |   
  |/   (_)  
  |         
  |         
  |         
__|_________"""
        elif self.errors == 5:
            message = u"""
  _________ 
  | /   |   
  |/        
  |         
  |         
  |         
__|_________"""
        elif self.errors == 4:
            message = u"""
  _________ 
  | /       
  |/        
  |         
  |         
  |         
__|_________"""
        elif self.errors == 3:
            message = u"""
  _________ 
  |         
  |         
  |         
  |         
  |        
__|_________"""
        elif self.errors == 2:
            message = u"""
            
  |         
  |         
  |         
  |         
  |         
__|_________"""
        elif self.errors == 1:
            message = u"""
            
            
            
            
            
            
____________"""
        elif self.errors == 0:
            message = u"""
            
            
            
            
            
            
            """
        else: message = u"""
  _________ 
  | /   |   
  |/   (_)  
  |         
  |    \O/  
  |     |   
__|____/_\__"""
        for line in message[1:].splitlines():
            c.privmsg(target, nick+u", "+line)

if __name__ == '__main__':
    Bot().start()
